import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'authentication.m.dart';


class AuthenticationService extends JsonRpcClientBase {
	AuthenticationService(
		String host, {
		String serviceName = "IAuthenticationService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => TokenInfo.fromJson(map));
		FJsonConvert.setDecoder((map) => AuthTokenDTO.fromJson(map));
	}

	Future<String> getAuthorizationAsync(AuthenticationRequest request) async {
		var rpcRst = await call("GetAuthorizationAsync", request);
		return rpcRst;
	}

	Future<TokenInfo> createTokenAsync(CreateTokenRequest request) async {
		var rpcRst = await call("CreateTokenAsync", request);
		var result = TokenInfo.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> removeTokenAsync(TokenRequest request) async {
		var rpcRst = await call("RemoveTokenAsync", request);
		return rpcRst;
	}

	Future<TokenInfo> getTokenInfoAsync(TokenRequest request) async {
		var rpcRst = await call("GetTokenInfoAsync", request);
		var result = TokenInfo.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> checkTokenAsync(TokenRequest request) async {
		var rpcRst = await call("CheckTokenAsync", request);
		return rpcRst;
	}

	Future<String> createAuthTokenAsync(CreateAuthTokenRequest request) async {
		var rpcRst = await call("CreateAuthTokenAsync", request);
		return rpcRst;
	}

	Future<AuthTokenDTO> getAuthTokenAsync(TokenRequest request) async {
		var rpcRst = await call("GetAuthTokenAsync", request);
		var result = AuthTokenDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<String> getServerDefaultTokenAsync() async {
		var rpcRst = await call("GetServerDefaultTokenAsync", );
		return rpcRst;
	}

}

