import 'package:fis_jsonrpc/utils.dart';

class BaseRequest {

	BaseRequest();

	factory BaseRequest.fromJson(Map<String, dynamic> map) {
		return BaseRequest( 
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		return map;
	}
}

class TokenRequest extends BaseRequest{
	String? token;

	TokenRequest({
		this.token,
	}) : super(
		);

	factory TokenRequest.fromJson(Map<String, dynamic> map) {
		return TokenRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(token != null)
			map['Token'] = token;
		return map;
	}
}

class AuthenticationRequest extends TokenRequest{
	String? fileName;

	AuthenticationRequest({
		this.fileName,
		String? token,
	}) : super(
			token: token,
		);

	factory AuthenticationRequest.fromJson(Map<String, dynamic> map) {
		return AuthenticationRequest( 
			fileName: map['FileName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(fileName != null)
			map['FileName'] = fileName;
		return map;
	}
}

class TokenInfo {
	String? userCode;
	String? deviceId;
	String? token;
	Map<String,String>? headerMap;

	TokenInfo({
		this.userCode,
		this.deviceId,
		this.token,
		this.headerMap,
	});

	factory TokenInfo.fromJson(Map<String, dynamic> map) {
		return TokenInfo( 
			userCode: map['UserCode'],
			deviceId: map['DeviceId'],
			token: map['Token'],
			headerMap: map['HeaderMap'] != null ? map['HeaderMap'].cast<String,String>() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(userCode != null)
			map['UserCode'] = userCode;
		if(deviceId != null)
			map['DeviceId'] = deviceId;
		if(token != null)
			map['Token'] = token;
		if(headerMap != null)
			map['HeaderMap'] = headerMap;
		return map;
	}
}

class TokenClientInfo {
	Map<String,String>? headerMap;

	TokenClientInfo({
		this.headerMap,
	});

	factory TokenClientInfo.fromJson(Map<String, dynamic> map) {
		return TokenClientInfo( 
			headerMap: map['HeaderMap'] != null ? map['HeaderMap'].cast<String,String>() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(headerMap != null)
			map['HeaderMap'] = headerMap;
		return map;
	}
}

class CreateTokenRequest {
	TokenClientInfo? clientInfo;
	String? userCode;

	CreateTokenRequest({
		this.clientInfo,
		this.userCode,
	});

	factory CreateTokenRequest.fromJson(Map<String, dynamic> map) {
		return CreateTokenRequest( 
			clientInfo: map['ClientInfo'] != null ? TokenClientInfo.fromJson(map['ClientInfo']) : null,
			userCode: map['UserCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(clientInfo != null)
			map['ClientInfo'] = clientInfo;
		if(userCode != null)
			map['UserCode'] = userCode;
		return map;
	}
}

class AuthTokenDTO {
	String? code;
	DateTime? issueTime;
	DateTime? expiryTime;
	String? sourceUrl;
	Map<String,String>? claims;

	AuthTokenDTO({
		this.code,
		this.issueTime,
		this.expiryTime,
		this.sourceUrl,
		this.claims,
	});

	factory AuthTokenDTO.fromJson(Map<String, dynamic> map) {
		return AuthTokenDTO( 
			code: map['Code'],
			issueTime: map['IssueTime'] != null ? DateTime.parse(map['IssueTime']) : null,
			expiryTime: map['ExpiryTime'] != null ? DateTime.parse(map['ExpiryTime']) : null,
			sourceUrl: map['SourceUrl'],
			claims: map['Claims'] != null ? map['Claims'].cast<String,String>() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(code != null)
			map['Code'] = code;
		if(issueTime != null)
			map['IssueTime'] = JsonRpcUtils.dateFormat(issueTime!);
		if(expiryTime != null)
			map['ExpiryTime'] = JsonRpcUtils.dateFormat(expiryTime!);
		if(sourceUrl != null)
			map['SourceUrl'] = sourceUrl;
		if(claims != null)
			map['Claims'] = claims;
		return map;
	}
}

class CreateAuthTokenRequest extends AuthTokenDTO{

	CreateAuthTokenRequest({
		String? code,
		DateTime? issueTime,
		DateTime? expiryTime,
		String? sourceUrl,
		Map<String,String>? claims,
	}) : super(
			code: code,
			issueTime: issueTime,
			expiryTime: expiryTime,
			sourceUrl: sourceUrl,
			claims: claims,
		);

	factory CreateAuthTokenRequest.fromJson(Map<String, dynamic> map) {
		return CreateAuthTokenRequest( 
			code: map['Code'],
			issueTime: map['IssueTime'] != null ? DateTime.parse(map['IssueTime']) : null,
			expiryTime: map['ExpiryTime'] != null ? DateTime.parse(map['ExpiryTime']) : null,
			sourceUrl: map['SourceUrl'],
			claims: map['Claims'] != null ? map['Claims'].cast<String,String>() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}


