import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'device.m.dart';

import 'authentication.m.dart';
import 'connect.m.dart';


class DeviceService extends JsonRpcClientBase {
	DeviceService(
		String host, {
		String serviceName = "IDeviceService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => DeviceInfoDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => DeviceExtendInfoDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => PageCollection<DeviceInfoDTO>.fromJson(map));
		FJsonConvert.setDecoder((map) => DictionaryDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => PageCollection<DeviceExtendInfoDTO>.fromJson(map));
		FJsonConvert.setDecoder((map) => SelectItemDTO.fromJson(map));
	}

	Future<DeviceInfoDTO> createDeviceInfoAsync(CreateDeviceRequest request) async {
		var rpcRst = await call("CreateDeviceInfoAsync", request);
		var result = DeviceInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<DeviceExtendInfoDTO> getDeviceInfoAsync(GetDeviceRequest request) async {
		var rpcRst = await call("GetDeviceInfoAsync", request);
		var result = DeviceExtendInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<DeviceExtendInfoDTO> getDeviceByShortCodeAsync(GetDeviceByShortCodeRequest request) async {
		var rpcRst = await call("GetDeviceByShortCodeAsync", request);
		var result = DeviceExtendInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<PageCollection<DeviceInfoDTO>> getDeviceInfoPageAsync(PageFilterRequest request) async {
		var rpcRst = await call("GetDeviceInfoPageAsync", request);
		var result = PageCollection<DeviceInfoDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> bindDeviceAsync(BindDeviceRequest request) async {
		var rpcRst = await call("BindDeviceAsync", request);
		return rpcRst;
	}

	Future<bool> modifyDeviceAsync(ModifyDeviceRequest request) async {
		var rpcRst = await call("ModifyDeviceAsync", request);
		return rpcRst;
	}

	Future<String> createDictionaryItemAsync(CreateDictionaryItemRequest request) async {
		var rpcRst = await call("CreateDictionaryItemAsync", request);
		return rpcRst;
	}

	Future<List<DictionaryDTO>> findDeviceModelItemsAsync(FindDeviceModelItemsRequest request) async {
		var rpcRst = await call("FindDeviceModelItemsAsync", request);
		var result = (rpcRst as List).map((e)=>DictionaryDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<List<DictionaryDTO>> findDeviceTypeItemsAsync(FindDeviceTypeItemsRequest request) async {
		var rpcRst = await call("FindDeviceTypeItemsAsync", request);
		var result = (rpcRst as List).map((e)=>DictionaryDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<bool> createShareDeviceToUserAsync(CreateShareDeviceToUserRequest request) async {
		var rpcRst = await call("CreateShareDeviceToUserAsync", request);
		return rpcRst;
	}

	Future<bool> deleteShareDeviceToUserAsync(DeleteShareDeviceToUserRequest request) async {
		var rpcRst = await call("DeleteShareDeviceToUserAsync", request);
		return rpcRst;
	}

	Future<bool> removeDeviceRelevancyAsync(RemoveDeviceRelevancyRequest request) async {
		var rpcRst = await call("RemoveDeviceRelevancyAsync", request);
		return rpcRst;
	}

	Future<PageCollection<DeviceExtendInfoDTO>> getPersonDeviceListAsync(GetPersonDeviceRequest request) async {
		var rpcRst = await call("GetPersonDeviceListAsync", request);
		var result = PageCollection<DeviceExtendInfoDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<DeviceExtendInfoDTO>> findDevicesByOrganizationCodeAsync(FindDevicesByOrganizationCodeRequest request) async {
		var rpcRst = await call("FindDevicesByOrganizationCodeAsync", request);
		var result = (rpcRst as List).map((e)=>DeviceExtendInfoDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<List<SelectItemDTO>> getPersonDeviceDropdownListAsync(TokenRequest request) async {
		var rpcRst = await call("GetPersonDeviceDropdownListAsync", request);
		var result = (rpcRst as List).map((e)=>SelectItemDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<Map<String,String>> queryServerConfigAsync(TokenRequest request) async {
		var rpcRst = await call("QueryServerConfigAsync", request);
		var result = (rpcRst as Map).cast<String,String>();
		return result;
	}

	Future<bool> cleanDeviceCacheAsync(CleanDeviceCacheRequest request) async {
		var rpcRst = await call("CleanDeviceCacheAsync", request);
		return rpcRst;
	}

	Future<bool> addDeviceToOrgAsync(AddDeviceToOrgRequest request) async {
		var rpcRst = await call("AddDeviceToOrgAsync", request);
		return rpcRst;
	}

	Future<List<String>> getDeviceBindUsersCodesAsync(GetDeviceRequest request) async {
		var rpcRst = await call("GetDeviceBindUsersCodesAsync", request);
		var result = (rpcRst as List).cast<String>().toList();
		return result;
	}

}

