enum MessageTypeEnum {
	Connection,
	Disconnect,
	Notify,
	KeepNotify,
	Ask,
	Ack,
}

enum NotificationTypeEnum {
	Message,
	Action,
	Update,
	NotifyRecordCodesMessage,
	FinishNotifyRecordsMessage,
}

class NotifyMessage {
	int length;
	MessageTypeEnum messageType;
	NotificationTypeEnum notificationType;
	String? message;

	NotifyMessage({
		this.length = 0,
		this.messageType = MessageTypeEnum.Connection,
		this.notificationType = NotificationTypeEnum.Message,
		this.message,
	});

	factory NotifyMessage.fromJson(Map<String, dynamic> map) {
		return NotifyMessage( 
			length: map['Length'],
			messageType: MessageTypeEnum.values.firstWhere((e) => e.index == map['MessageType']),
			notificationType: NotificationTypeEnum.values.firstWhere((e) => e.index == map['NotificationType']),
			message: map['Message'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Length'] = length;
		map['MessageType'] = messageType.index;
		map['NotificationType'] = notificationType.index;
		if(message != null)
			map['Message'] = message;
		return map;
	}
}

class SendNotificationRequest {
	String? userCode;
	NotifyMessage? message;

	SendNotificationRequest({
		this.userCode,
		this.message,
	});

	factory SendNotificationRequest.fromJson(Map<String, dynamic> map) {
		return SendNotificationRequest( 
			userCode: map['UserCode'],
			message: map['Message'] != null ? NotifyMessage.fromJson(map['Message']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(userCode != null)
			map['UserCode'] = userCode;
		if(message != null)
			map['Message'] = message;
		return map;
	}
}

class SubscribeNotifyRequest {
	String? userCode;
	String? token;

	SubscribeNotifyRequest({
		this.userCode,
		this.token,
	});

	factory SubscribeNotifyRequest.fromJson(Map<String, dynamic> map) {
		return SubscribeNotifyRequest( 
			userCode: map['UserCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(userCode != null)
			map['UserCode'] = userCode;
		if(token != null)
			map['Token'] = token;
		return map;
	}
}


