import 'connect.m.dart';
import 'authentication.m.dart';

class OrganizationBaseDTO extends BaseDTO{
	String? organizationCode;
	String? organizationName;

	OrganizationBaseDTO({
		this.organizationCode,
		this.organizationName,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory OrganizationBaseDTO.fromJson(Map<String, dynamic> map) {
		return OrganizationBaseDTO( 
			organizationCode: map['OrganizationCode'],
			organizationName: map['OrganizationName'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		return map;
	}
}

enum OrganizationPatientTypeEnum {
	Person,
	Animals,
}

class OrganizationBasicDTO extends OrganizationBaseDTO{
	String? regionCode;
	String? parentCode;
	String? logoUrl;
	OrganizationPatientTypeEnum patientType;
	bool isinvented;

	OrganizationBasicDTO({
		this.regionCode,
		this.parentCode,
		this.logoUrl,
		this.patientType = OrganizationPatientTypeEnum.Person,
		this.isinvented = false,
		String? organizationCode,
		String? organizationName,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			organizationCode: organizationCode,
			organizationName: organizationName,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory OrganizationBasicDTO.fromJson(Map<String, dynamic> map) {
		return OrganizationBasicDTO( 
			regionCode: map['RegionCode'],
			parentCode: map['ParentCode'],
			logoUrl: map['LogoUrl'],
			patientType: OrganizationPatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
			isinvented: map['Isinvented'],
			organizationCode: map['OrganizationCode'],
			organizationName: map['OrganizationName'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(regionCode != null)
			map['RegionCode'] = regionCode;
		if(parentCode != null)
			map['ParentCode'] = parentCode;
		if(logoUrl != null)
			map['LogoUrl'] = logoUrl;
		map['PatientType'] = patientType.index;
		map['Isinvented'] = isinvented;
		return map;
	}
}

enum OrganizationTypeEnum {
	Corporation,
	Department,
	Section,
}

enum OrganizationStateEnum {
	WaitAudit,
	Audited,
	Rejected,
}

class OrganizationDTO extends OrganizationBasicDTO{
	String? description;
	String? rootCode;
	OrganizationTypeEnum organizationType;
	List<String>? authorityGroups;
	String? nautica;
	OrganizationStateEnum state;
	List<String>? directors;
	List<String>? assignedAdmins;
	String? patientSettingJson;
	String? examSettingJson;
	String? patientSettingVersion;
	String? examSettingVersion;

	OrganizationDTO({
		this.description,
		this.rootCode,
		this.organizationType = OrganizationTypeEnum.Corporation,
		this.authorityGroups,
		this.nautica,
		this.state = OrganizationStateEnum.WaitAudit,
		this.directors,
		this.assignedAdmins,
		this.patientSettingJson,
		this.examSettingJson,
		this.patientSettingVersion,
		this.examSettingVersion,
		String? regionCode,
		String? parentCode,
		String? logoUrl,
		OrganizationPatientTypeEnum patientType = OrganizationPatientTypeEnum.Person,
		bool isinvented = false,
		String? organizationCode,
		String? organizationName,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			regionCode: regionCode,
			parentCode: parentCode,
			logoUrl: logoUrl,
			patientType: patientType,
			isinvented: isinvented,
			organizationCode: organizationCode,
			organizationName: organizationName,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory OrganizationDTO.fromJson(Map<String, dynamic> map) {
		return OrganizationDTO( 
			description: map['Description'],
			rootCode: map['RootCode'],
			organizationType: OrganizationTypeEnum.values.firstWhere((e) => e.index == map['OrganizationType']),
			authorityGroups: map['AuthorityGroups'] != null ? map['AuthorityGroups'].cast<String>().toList() : null,
			nautica: map['Nautica'],
			state: OrganizationStateEnum.values.firstWhere((e) => e.index == map['State']),
			directors: map['Directors'] != null ? map['Directors'].cast<String>().toList() : null,
			assignedAdmins: map['AssignedAdmins'] != null ? map['AssignedAdmins'].cast<String>().toList() : null,
			patientSettingJson: map['PatientSettingJson'],
			examSettingJson: map['ExamSettingJson'],
			patientSettingVersion: map['PatientSettingVersion'],
			examSettingVersion: map['ExamSettingVersion'],
			regionCode: map['RegionCode'],
			parentCode: map['ParentCode'],
			logoUrl: map['LogoUrl'],
			patientType: OrganizationPatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
			isinvented: map['Isinvented'],
			organizationCode: map['OrganizationCode'],
			organizationName: map['OrganizationName'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(description != null)
			map['Description'] = description;
		if(rootCode != null)
			map['RootCode'] = rootCode;
		map['OrganizationType'] = organizationType.index;
		if(authorityGroups != null)
			map['AuthorityGroups'] = authorityGroups;
		if(nautica != null)
			map['Nautica'] = nautica;
		map['State'] = state.index;
		if(directors != null)
			map['Directors'] = directors;
		if(assignedAdmins != null)
			map['AssignedAdmins'] = assignedAdmins;
		if(patientSettingJson != null)
			map['PatientSettingJson'] = patientSettingJson;
		if(examSettingJson != null)
			map['ExamSettingJson'] = examSettingJson;
		if(patientSettingVersion != null)
			map['PatientSettingVersion'] = patientSettingVersion;
		if(examSettingVersion != null)
			map['ExamSettingVersion'] = examSettingVersion;
		return map;
	}
}

class SearchOrganizationsRequest extends TokenRequest{
	String? keyword;
	String? parentCode;
	OrganizationTypeEnum organizationType;

	SearchOrganizationsRequest({
		this.keyword,
		this.parentCode,
		this.organizationType = OrganizationTypeEnum.Corporation,
		String? token,
	}) : super(
			token: token,
		);

	factory SearchOrganizationsRequest.fromJson(Map<String, dynamic> map) {
		return SearchOrganizationsRequest( 
			keyword: map['Keyword'],
			parentCode: map['ParentCode'],
			organizationType: OrganizationTypeEnum.values.firstWhere((e) => e.index == map['OrganizationType']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(keyword != null)
			map['Keyword'] = keyword;
		if(parentCode != null)
			map['ParentCode'] = parentCode;
		map['OrganizationType'] = organizationType.index;
		return map;
	}
}

class GetOrganizationByCodeRequest extends TokenRequest{
	String? organizationCode;

	GetOrganizationByCodeRequest({
		this.organizationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetOrganizationByCodeRequest.fromJson(Map<String, dynamic> map) {
		return GetOrganizationByCodeRequest( 
			organizationCode: map['OrganizationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		return map;
	}
}

class GetParentOrganizationListRequest extends TokenRequest{

	GetParentOrganizationListRequest({
		String? token,
	}) : super(
			token: token,
		);

	factory GetParentOrganizationListRequest.fromJson(Map<String, dynamic> map) {
		return GetParentOrganizationListRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class GetPersonOrganizationRequest extends TokenRequest{

	GetPersonOrganizationRequest({
		String? token,
	}) : super(
			token: token,
		);

	factory GetPersonOrganizationRequest.fromJson(Map<String, dynamic> map) {
		return GetPersonOrganizationRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class SavePersonOrganizationRequest {
	String? token;
	String? regionCode;
	String? parentCode;
	String? logoUrl;
	String? organizationCode;
	String? organizationName;
	OrganizationPatientTypeEnum patientType;

	SavePersonOrganizationRequest({
		this.token,
		this.regionCode,
		this.parentCode,
		this.logoUrl,
		this.organizationCode,
		this.organizationName,
		this.patientType = OrganizationPatientTypeEnum.Person,
	});

	factory SavePersonOrganizationRequest.fromJson(Map<String, dynamic> map) {
		return SavePersonOrganizationRequest( 
			token: map['Token'],
			regionCode: map['RegionCode'],
			parentCode: map['ParentCode'],
			logoUrl: map['LogoUrl'],
			organizationCode: map['OrganizationCode'],
			organizationName: map['OrganizationName'],
			patientType: OrganizationPatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(token != null)
			map['Token'] = token;
		if(regionCode != null)
			map['RegionCode'] = regionCode;
		if(parentCode != null)
			map['ParentCode'] = parentCode;
		if(logoUrl != null)
			map['LogoUrl'] = logoUrl;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		map['PatientType'] = patientType.index;
		return map;
	}
}

class OrganizationItemDTO {
	String? organizationName;
	String? parentCode;
	String? extendsData;
	OrganizationPatientTypeEnum patientType;

	OrganizationItemDTO({
		this.organizationName,
		this.parentCode,
		this.extendsData,
		this.patientType = OrganizationPatientTypeEnum.Person,
	});

	factory OrganizationItemDTO.fromJson(Map<String, dynamic> map) {
		return OrganizationItemDTO( 
			organizationName: map['OrganizationName'],
			parentCode: map['ParentCode'],
			extendsData: map['ExtendsData'],
			patientType: OrganizationPatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		if(parentCode != null)
			map['ParentCode'] = parentCode;
		if(extendsData != null)
			map['ExtendsData'] = extendsData;
		map['PatientType'] = patientType.index;
		return map;
	}
}

class AddOrganizationsRequest extends TokenRequest{
	List<OrganizationItemDTO>? organizationInfos;

	AddOrganizationsRequest({
		this.organizationInfos,
		String? token,
	}) : super(
			token: token,
		);

	factory AddOrganizationsRequest.fromJson(Map<String, dynamic> map) {
		return AddOrganizationsRequest( 
			organizationInfos: map['OrganizationInfos'] != null ? (map['OrganizationInfos'] as List).map((e)=>OrganizationItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationInfos != null)
			map['OrganizationInfos'] = organizationInfos;
		return map;
	}
}

class RemoveOrganizationsRequest extends TokenRequest{
	List<String>? organizationCodes;

	RemoveOrganizationsRequest({
		this.organizationCodes,
		String? token,
	}) : super(
			token: token,
		);

	factory RemoveOrganizationsRequest.fromJson(Map<String, dynamic> map) {
		return RemoveOrganizationsRequest( 
			organizationCodes: map['OrganizationCodes'] != null ? map['OrganizationCodes'].cast<String>().toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCodes != null)
			map['OrganizationCodes'] = organizationCodes;
		return map;
	}
}

class UpdateOrganizationNameRequest extends TokenRequest{
	String? organizationCode;
	String? organizationName;

	UpdateOrganizationNameRequest({
		this.organizationCode,
		this.organizationName,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateOrganizationNameRequest.fromJson(Map<String, dynamic> map) {
		return UpdateOrganizationNameRequest( 
			organizationCode: map['OrganizationCode'],
			organizationName: map['OrganizationName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		return map;
	}
}

class OrganizationSettingResult {
	String? settingVersion;
	String? settingData;

	OrganizationSettingResult({
		this.settingVersion,
		this.settingData,
	});

	factory OrganizationSettingResult.fromJson(Map<String, dynamic> map) {
		return OrganizationSettingResult( 
			settingVersion: map['SettingVersion'],
			settingData: map['SettingData'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(settingVersion != null)
			map['SettingVersion'] = settingVersion;
		if(settingData != null)
			map['SettingData'] = settingData;
		return map;
	}
}

enum OrganizationSettingTypeEnum {
	Patient,
	Exam,
}

class GetUserOrganizationSettingRequest extends TokenRequest{
	OrganizationSettingTypeEnum settingType;
	String? version;

	GetUserOrganizationSettingRequest({
		this.settingType = OrganizationSettingTypeEnum.Patient,
		this.version,
		String? token,
	}) : super(
			token: token,
		);

	factory GetUserOrganizationSettingRequest.fromJson(Map<String, dynamic> map) {
		return GetUserOrganizationSettingRequest( 
			settingType: OrganizationSettingTypeEnum.values.firstWhere((e) => e.index == map['SettingType']),
			version: map['Version'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['SettingType'] = settingType.index;
		if(version != null)
			map['Version'] = version;
		return map;
	}
}

enum ServerLangugeSettingEnum {
	ZH,
	EN,
}

class GetServerLangugeSettingRequest extends TokenRequest{
	ServerLangugeSettingEnum settingType;
	String? version;

	GetServerLangugeSettingRequest({
		this.settingType = ServerLangugeSettingEnum.ZH,
		this.version,
		String? token,
	}) : super(
			token: token,
		);

	factory GetServerLangugeSettingRequest.fromJson(Map<String, dynamic> map) {
		return GetServerLangugeSettingRequest( 
			settingType: ServerLangugeSettingEnum.values.firstWhere((e) => e.index == map['SettingType']),
			version: map['Version'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['SettingType'] = settingType.index;
		if(version != null)
			map['Version'] = version;
		return map;
	}
}


