import 'authentication.m.dart';
import 'patient.m.dart';
import 'device.m.dart';
import 'organization.m.dart';

import 'package:fis_jsonrpc/utils.dart';

class PatientInfoExt {
	String? patientScanType;
	List<DataItemDTO>? content;

	PatientInfoExt({
		this.patientScanType,
		this.content,
	});

	factory PatientInfoExt.fromJson(Map<String, dynamic> map) {
		return PatientInfoExt( 
			patientScanType: map['PatientScanType'],
			content: map['Content'] != null ? (map['Content'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(patientScanType != null)
			map['PatientScanType'] = patientScanType;
		if(content != null)
			map['Content'] = content;
		return map;
	}
}

class CreateRecordRequest extends TokenRequest{
	String? patientCode;
	String? deviceCode;
	List<PatientInfoExt>? patientInfoExtList;

	CreateRecordRequest({
		this.patientCode,
		this.deviceCode,
		this.patientInfoExtList,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateRecordRequest.fromJson(Map<String, dynamic> map) {
		return CreateRecordRequest( 
			patientCode: map['PatientCode'],
			deviceCode: map['DeviceCode'],
			patientInfoExtList: map['PatientInfoExtList'] != null ? (map['PatientInfoExtList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(patientInfoExtList != null)
			map['PatientInfoExtList'] = patientInfoExtList;
		return map;
	}
}

enum RecordStatusEnum {
	NotScanned,
	NotReport,
	Completed,
}

class GetRecordsPageResult {
	DateTime? createTime;
	String? deptName;
	String? creatorName;
	String? deviceName;
	String? reportNum;
	String? recordCode;
	RecordStatusEnum recordStatus;
	bool isRead;

	GetRecordsPageResult({
		this.createTime,
		this.deptName,
		this.creatorName,
		this.deviceName,
		this.reportNum,
		this.recordCode,
		this.recordStatus = RecordStatusEnum.NotScanned,
		this.isRead = false,
	});

	factory GetRecordsPageResult.fromJson(Map<String, dynamic> map) {
		return GetRecordsPageResult( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			deptName: map['DeptName'],
			creatorName: map['CreatorName'],
			deviceName: map['DeviceName'],
			reportNum: map['ReportNum'],
			recordCode: map['RecordCode'],
			recordStatus: RecordStatusEnum.values.firstWhere((e) => e.index == map['RecordStatus']),
			isRead: map['IsRead'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(deptName != null)
			map['DeptName'] = deptName;
		if(creatorName != null)
			map['CreatorName'] = creatorName;
		if(deviceName != null)
			map['DeviceName'] = deviceName;
		if(reportNum != null)
			map['ReportNum'] = reportNum;
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		map['RecordStatus'] = recordStatus.index;
		map['IsRead'] = isRead;
		return map;
	}
}

enum QueryRecordStatusEnum {
	All,
	NotScanned,
	NotReport,
	Completed,
	NotCompleted,
}

enum QueryRecordCreateTypeEnum {
	All,
	Reservation,
	Normal,
}

class GetRecordsPageRequest extends PageRequest{
	String? patientCode;
	QueryRecordStatusEnum queryRecordStatus;
	QueryRecordCreateTypeEnum queryRecordCreateType;

	GetRecordsPageRequest({
		this.patientCode,
		this.queryRecordStatus = QueryRecordStatusEnum.All,
		this.queryRecordCreateType = QueryRecordCreateTypeEnum.All,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory GetRecordsPageRequest.fromJson(Map<String, dynamic> map) {
		return GetRecordsPageRequest( 
			patientCode: map['PatientCode'],
			queryRecordStatus: QueryRecordStatusEnum.values.firstWhere((e) => e.index == map['QueryRecordStatus']),
			queryRecordCreateType: QueryRecordCreateTypeEnum.values.firstWhere((e) => e.index == map['QueryRecordCreateType']),
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		map['QueryRecordStatus'] = queryRecordStatus.index;
		map['QueryRecordCreateType'] = queryRecordCreateType.index;
		return map;
	}
}

class QueryRecordResult {
	DateTime? createTime;
	String? deptName;
	String? patientName;
	String? patientAge;
	int patientSex;
	String? creatorName;
	String? deviceName;
	RecordStatusEnum recordStatus;
	List<PatientInfoExt>? patientInfoExtList;

	QueryRecordResult({
		this.createTime,
		this.deptName,
		this.patientName,
		this.patientAge,
		this.patientSex = 0,
		this.creatorName,
		this.deviceName,
		this.recordStatus = RecordStatusEnum.NotScanned,
		this.patientInfoExtList,
	});

	factory QueryRecordResult.fromJson(Map<String, dynamic> map) {
		return QueryRecordResult( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			deptName: map['DeptName'],
			patientName: map['PatientName'],
			patientAge: map['PatientAge'],
			patientSex: map['PatientSex'],
			creatorName: map['CreatorName'],
			deviceName: map['DeviceName'],
			recordStatus: RecordStatusEnum.values.firstWhere((e) => e.index == map['RecordStatus']),
			patientInfoExtList: map['PatientInfoExtList'] != null ? (map['PatientInfoExtList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(deptName != null)
			map['DeptName'] = deptName;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(patientAge != null)
			map['PatientAge'] = patientAge;
		map['PatientSex'] = patientSex;
		if(creatorName != null)
			map['CreatorName'] = creatorName;
		if(deviceName != null)
			map['DeviceName'] = deviceName;
		map['RecordStatus'] = recordStatus.index;
		if(patientInfoExtList != null)
			map['PatientInfoExtList'] = patientInfoExtList;
		return map;
	}
}

class QueryRecordRequest extends TokenRequest{
	String? recordCode;

	QueryRecordRequest({
		this.recordCode,
		String? token,
	}) : super(
			token: token,
		);

	factory QueryRecordRequest.fromJson(Map<String, dynamic> map) {
		return QueryRecordRequest( 
			recordCode: map['RecordCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		return map;
	}
}

class PushRecordCodesToDeviceRequest extends TokenRequest{
	List<String>? recordCodes;
	String? deviceCode;

	PushRecordCodesToDeviceRequest({
		this.recordCodes,
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory PushRecordCodesToDeviceRequest.fromJson(Map<String, dynamic> map) {
		return PushRecordCodesToDeviceRequest( 
			recordCodes: map['RecordCodes'] != null ? map['RecordCodes'].cast<String>().toList() : null,
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(recordCodes != null)
			map['RecordCodes'] = recordCodes;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class ProcessRecordDataResult {
	List<DataItemDTO>? content;

	ProcessRecordDataResult({
		this.content,
	});

	factory ProcessRecordDataResult.fromJson(Map<String, dynamic> map) {
		return ProcessRecordDataResult( 
			content: map['Content'] != null ? (map['Content'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(content != null)
			map['Content'] = content;
		return map;
	}
}

enum AnimalSpeciesEnum {
	Bovine,
	Canidae,
	Equidae,
	Felidae,
	Caprinae,
	Suidae,
}

class ProcessRecordDataRequest extends TokenRequest{
	String? methodName;
	List<DataItemDTO>? content;
	OrganizationPatientTypeEnum patientType;
	AnimalSpeciesEnum speciesEnum;

	ProcessRecordDataRequest({
		this.methodName,
		this.content,
		this.patientType = OrganizationPatientTypeEnum.Person,
		this.speciesEnum = AnimalSpeciesEnum.Bovine,
		String? token,
	}) : super(
			token: token,
		);

	factory ProcessRecordDataRequest.fromJson(Map<String, dynamic> map) {
		return ProcessRecordDataRequest( 
			methodName: map['MethodName'],
			content: map['Content'] != null ? (map['Content'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			patientType: OrganizationPatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
			speciesEnum: AnimalSpeciesEnum.values.firstWhere((e) => e.index == map['SpeciesEnum']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(methodName != null)
			map['MethodName'] = methodName;
		if(content != null)
			map['Content'] = content;
		map['PatientType'] = patientType.index;
		map['SpeciesEnum'] = speciesEnum.index;
		return map;
	}
}

class RelevanceRecordRequest extends TokenRequest{
	String? examCode;
	String? rservationCode;

	RelevanceRecordRequest({
		this.examCode,
		this.rservationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RelevanceRecordRequest.fromJson(Map<String, dynamic> map) {
		return RelevanceRecordRequest( 
			examCode: map['ExamCode'],
			rservationCode: map['RservationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(examCode != null)
			map['ExamCode'] = examCode;
		if(rservationCode != null)
			map['RservationCode'] = rservationCode;
		return map;
	}
}


