import 'authentication.m.dart';
import 'patient.m.dart';
import 'recordInfo.m.dart';
import 'connect.m.dart';
import 'device.m.dart';
import 'organization.m.dart';

import 'package:fis_jsonrpc/utils.dart';

class CreateExaminfoResult {
	String? examCode;

	CreateExaminfoResult({
		this.examCode,
	});

	factory CreateExaminfoResult.fromJson(Map<String, dynamic> map) {
		return CreateExaminfoResult( 
			examCode: map['ExamCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(examCode != null)
			map['ExamCode'] = examCode;
		return map;
	}
}

class CreateExaminfoRequest extends TokenRequest{
	String? patientType;
	String? reservationCode;
	List<DataItemDTO>? patientInfo;
	List<PatientInfoExt>? patientScanInfoList;

	CreateExaminfoRequest({
		this.patientType,
		this.reservationCode,
		this.patientInfo,
		this.patientScanInfoList,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateExaminfoRequest.fromJson(Map<String, dynamic> map) {
		return CreateExaminfoRequest( 
			patientType: map['PatientType'],
			reservationCode: map['ReservationCode'],
			patientInfo: map['PatientInfo'] != null ? (map['PatientInfo'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			patientScanInfoList: map['PatientScanInfoList'] != null ? (map['PatientScanInfoList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientType != null)
			map['PatientType'] = patientType;
		if(reservationCode != null)
			map['ReservationCode'] = reservationCode;
		if(patientInfo != null)
			map['PatientInfo'] = patientInfo;
		if(patientScanInfoList != null)
			map['PatientScanInfoList'] = patientScanInfoList;
		return map;
	}
}

class UploadExamDataRequest extends TokenRequest{
	String? examCode;
	String? previewFileToken;
	String? fileToken;
	String? application;
	String? patientScanType;

	UploadExamDataRequest({
		this.examCode,
		this.previewFileToken,
		this.fileToken,
		this.application,
		this.patientScanType,
		String? token,
	}) : super(
			token: token,
		);

	factory UploadExamDataRequest.fromJson(Map<String, dynamic> map) {
		return UploadExamDataRequest( 
			examCode: map['ExamCode'],
			previewFileToken: map['PreviewFileToken'],
			fileToken: map['FileToken'],
			application: map['Application'],
			patientScanType: map['PatientScanType'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(examCode != null)
			map['ExamCode'] = examCode;
		if(previewFileToken != null)
			map['PreviewFileToken'] = previewFileToken;
		if(fileToken != null)
			map['FileToken'] = fileToken;
		if(application != null)
			map['Application'] = application;
		if(patientScanType != null)
			map['PatientScanType'] = patientScanType;
		return map;
	}
}

class TerminalImageDTO {
	String? previewUrl;
	String? imageUrl;

	TerminalImageDTO({
		this.previewUrl,
		this.imageUrl,
	});

	factory TerminalImageDTO.fromJson(Map<String, dynamic> map) {
		return TerminalImageDTO( 
			previewUrl: map['PreviewUrl'],
			imageUrl: map['ImageUrl'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(previewUrl != null)
			map['PreviewUrl'] = previewUrl;
		if(imageUrl != null)
			map['ImageUrl'] = imageUrl;
		return map;
	}
}

enum RemedicalFileDataTypeEnum {
	VinnoVidSingle,
	ThirdVidSingle,
	VinnoVidMovie,
	ThirdVidMovie,
	Image,
}

enum RemedicalImageLocationTypeEnum {
	BreastImageLocation,
	LiverImageLocation,
	ThyroidImageLocation,
	PositionImageLocation,
}

enum RemedicalImagePositionTypeEnum {
	None,
	Left,
	Right,
	Middle,
}

enum RemedicalApparatusTypeEnum {
	BreastOuterUpper,
	BreastInnerUpper,
	BreastOuterLower,
	BreastInnerLower,
	Lobe,
	LobeOfUnder,
	LobeOfInter,
	Thyroid,
	Abdomen,
}

class ImageLocationDTO {
	RemedicalImageLocationTypeEnum imageLocationType;
	RemedicalImagePositionTypeEnum position;
	String? scanLocationName;
	RemedicalApparatusTypeEnum apparatusType;

	ImageLocationDTO({
		this.imageLocationType = RemedicalImageLocationTypeEnum.BreastImageLocation,
		this.position = RemedicalImagePositionTypeEnum.None,
		this.scanLocationName,
		this.apparatusType = RemedicalApparatusTypeEnum.BreastOuterUpper,
	});

	factory ImageLocationDTO.fromJson(Map<String, dynamic> map) {
		return ImageLocationDTO( 
			imageLocationType: RemedicalImageLocationTypeEnum.values.firstWhere((e) => e.index == map['ImageLocationType']),
			position: RemedicalImagePositionTypeEnum.values.firstWhere((e) => e.index == map['Position']),
			scanLocationName: map['ScanLocationName'],
			apparatusType: RemedicalApparatusTypeEnum.values.firstWhere((e) => e.index == map['ApparatusType']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['ImageLocationType'] = imageLocationType.index;
		map['Position'] = position.index;
		if(scanLocationName != null)
			map['ScanLocationName'] = scanLocationName;
		map['ApparatusType'] = apparatusType.index;
		return map;
	}
}

enum RemedicalAIDiagnosisStatusEnum {
	Null,
	NoObviousLesion,
	Benign,
	Malignant,
	BenignAndMalignant,
}

class RemedicalInfoDTO extends BaseDTO{
	String? remedicalCode;
	String? deviceCode;
	String? recordCode;
	String? patientScanType;
	String? application;
	TerminalImageDTO? terminalImages;
	RemedicalFileDataTypeEnum fileDataType;
	ImageLocationDTO? imageLocation;
	RemedicalAIDiagnosisStatusEnum aIDiagnosisStatus;

	RemedicalInfoDTO({
		this.remedicalCode,
		this.deviceCode,
		this.recordCode,
		this.patientScanType,
		this.application,
		this.terminalImages,
		this.fileDataType = RemedicalFileDataTypeEnum.VinnoVidSingle,
		this.imageLocation,
		this.aIDiagnosisStatus = RemedicalAIDiagnosisStatusEnum.Null,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory RemedicalInfoDTO.fromJson(Map<String, dynamic> map) {
		return RemedicalInfoDTO( 
			remedicalCode: map['RemedicalCode'],
			deviceCode: map['DeviceCode'],
			recordCode: map['RecordCode'],
			patientScanType: map['PatientScanType'],
			application: map['Application'],
			terminalImages: map['TerminalImages'] != null ? TerminalImageDTO.fromJson(map['TerminalImages']) : null,
			fileDataType: RemedicalFileDataTypeEnum.values.firstWhere((e) => e.index == map['FileDataType']),
			imageLocation: map['ImageLocation'] != null ? ImageLocationDTO.fromJson(map['ImageLocation']) : null,
			aIDiagnosisStatus: RemedicalAIDiagnosisStatusEnum.values.firstWhere((e) => e.index == map['AIDiagnosisStatus']),
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(remedicalCode != null)
			map['RemedicalCode'] = remedicalCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		if(patientScanType != null)
			map['PatientScanType'] = patientScanType;
		if(application != null)
			map['Application'] = application;
		if(terminalImages != null)
			map['TerminalImages'] = terminalImages;
		map['FileDataType'] = fileDataType.index;
		if(imageLocation != null)
			map['ImageLocation'] = imageLocation;
		map['AIDiagnosisStatus'] = aIDiagnosisStatus.index;
		return map;
	}
}

class RemedicalItemList {
	String? patientScanTypeDesc;
	List<RemedicalInfoDTO>? remedicalList;

	RemedicalItemList({
		this.patientScanTypeDesc,
		this.remedicalList,
	});

	factory RemedicalItemList.fromJson(Map<String, dynamic> map) {
		return RemedicalItemList( 
			patientScanTypeDesc: map['PatientScanTypeDesc'],
			remedicalList: map['RemedicalList'] != null ? (map['RemedicalList'] as List).map((e)=>RemedicalInfoDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(patientScanTypeDesc != null)
			map['PatientScanTypeDesc'] = patientScanTypeDesc;
		if(remedicalList != null)
			map['RemedicalList'] = remedicalList;
		return map;
	}
}

class RemedicalListResult {
	String? sacnDate;
	String? recordCode;
	List<RemedicalItemList>? remedicalItemList;

	RemedicalListResult({
		this.sacnDate,
		this.recordCode,
		this.remedicalItemList,
	});

	factory RemedicalListResult.fromJson(Map<String, dynamic> map) {
		return RemedicalListResult( 
			sacnDate: map['SacnDate'],
			recordCode: map['RecordCode'],
			remedicalItemList: map['RemedicalItemList'] != null ? (map['RemedicalItemList'] as List).map((e)=>RemedicalItemList.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(sacnDate != null)
			map['SacnDate'] = sacnDate;
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		if(remedicalItemList != null)
			map['RemedicalItemList'] = remedicalItemList;
		return map;
	}
}

enum QueryDropdownListEnum {
	Org,
}

class QueryDropdownListReuqest extends TokenRequest{
	QueryDropdownListEnum queryType;
	String? queryValue;

	QueryDropdownListReuqest({
		this.queryType = QueryDropdownListEnum.Org,
		this.queryValue,
		String? token,
	}) : super(
			token: token,
		);

	factory QueryDropdownListReuqest.fromJson(Map<String, dynamic> map) {
		return QueryDropdownListReuqest( 
			queryType: QueryDropdownListEnum.values.firstWhere((e) => e.index == map['QueryType']),
			queryValue: map['QueryValue'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['QueryType'] = queryType.index;
		if(queryValue != null)
			map['QueryValue'] = queryValue;
		return map;
	}
}

class RservationResult {
	String? reservationCode;
	List<DataItemDTO>? patientInfo;
	List<PatientInfoExt>? patientInfoExtList;
	String? dataSource;

	RservationResult({
		this.reservationCode,
		this.patientInfo,
		this.patientInfoExtList,
		this.dataSource,
	});

	factory RservationResult.fromJson(Map<String, dynamic> map) {
		return RservationResult( 
			reservationCode: map['ReservationCode'],
			patientInfo: map['PatientInfo'] != null ? (map['PatientInfo'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			patientInfoExtList: map['PatientInfoExtList'] != null ? (map['PatientInfoExtList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			dataSource: map['DataSource'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(reservationCode != null)
			map['ReservationCode'] = reservationCode;
		if(patientInfo != null)
			map['PatientInfo'] = patientInfo;
		if(patientInfoExtList != null)
			map['PatientInfoExtList'] = patientInfoExtList;
		if(dataSource != null)
			map['DataSource'] = dataSource;
		return map;
	}
}

class QueryReservationResult {
	List<RservationResult>? reservationList;

	QueryReservationResult({
		this.reservationList,
	});

	factory QueryReservationResult.fromJson(Map<String, dynamic> map) {
		return QueryReservationResult( 
			reservationList: map['ReservationList'] != null ? (map['ReservationList'] as List).map((e)=>RservationResult.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(reservationList != null)
			map['ReservationList'] = reservationList;
		return map;
	}
}

class QueryReservationRequest extends TokenRequest{
	DateTime? createTime;

	QueryReservationRequest({
		this.createTime,
		String? token,
	}) : super(
			token: token,
		);

	factory QueryReservationRequest.fromJson(Map<String, dynamic> map) {
		return QueryReservationRequest( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		return map;
	}
}

class FinishExamNotifyDetail {
	String? recordCode;
	RecordStatusEnum recordStatus;

	FinishExamNotifyDetail({
		this.recordCode,
		this.recordStatus = RecordStatusEnum.NotScanned,
	});

	factory FinishExamNotifyDetail.fromJson(Map<String, dynamic> map) {
		return FinishExamNotifyDetail( 
			recordCode: map['RecordCode'],
			recordStatus: RecordStatusEnum.values.firstWhere((e) => e.index == map['RecordStatus']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		map['RecordStatus'] = recordStatus.index;
		return map;
	}
}

class PushFinishExamNotifyToClientRequest {
	List<FinishExamNotifyDetail>? records;
	String? userCode;

	PushFinishExamNotifyToClientRequest({
		this.records,
		this.userCode,
	});

	factory PushFinishExamNotifyToClientRequest.fromJson(Map<String, dynamic> map) {
		return PushFinishExamNotifyToClientRequest( 
			records: map['Records'] != null ? (map['Records'] as List).map((e)=>FinishExamNotifyDetail.fromJson(e as Map<String,dynamic>)).toList() : null,
			userCode: map['UserCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(records != null)
			map['Records'] = records;
		if(userCode != null)
			map['UserCode'] = userCode;
		return map;
	}
}

class DeviceFinishExamRequest extends TokenRequest{
	List<String>? records;

	DeviceFinishExamRequest({
		this.records,
		String? token,
	}) : super(
			token: token,
		);

	factory DeviceFinishExamRequest.fromJson(Map<String, dynamic> map) {
		return DeviceFinishExamRequest( 
			records: map['Records'] != null ? map['Records'].cast<String>().toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(records != null)
			map['Records'] = records;
		return map;
	}
}

enum ThesaurusStatusTypeEnum {
	Unpublished,
	Published,
	Referenced,
}

enum ThesaurusTypeEnum {
	Default,
	Organization,
	Personal,
}

enum ThesaurusUsageTypeEnum {
	People,
	Animal,
}

class ThesaurusDTO {
	String? thesaurusCode;
	String? thesaurusName;
	ThesaurusStatusTypeEnum thesaurusStateType;
	ThesaurusTypeEnum thesaurusType;
	ThesaurusUsageTypeEnum thesaurusUsageType;
	String? organizationCode;
	String? personalCode;
	String? languageCode;
	List<String>? thesaurusUser;
	bool isDefault;

	ThesaurusDTO({
		this.thesaurusCode,
		this.thesaurusName,
		this.thesaurusStateType = ThesaurusStatusTypeEnum.Unpublished,
		this.thesaurusType = ThesaurusTypeEnum.Default,
		this.thesaurusUsageType = ThesaurusUsageTypeEnum.People,
		this.organizationCode,
		this.personalCode,
		this.languageCode,
		this.thesaurusUser,
		this.isDefault = false,
	});

	factory ThesaurusDTO.fromJson(Map<String, dynamic> map) {
		return ThesaurusDTO( 
			thesaurusCode: map['ThesaurusCode'],
			thesaurusName: map['ThesaurusName'],
			thesaurusStateType: ThesaurusStatusTypeEnum.values.firstWhere((e) => e.index == map['ThesaurusStateType']),
			thesaurusType: ThesaurusTypeEnum.values.firstWhere((e) => e.index == map['ThesaurusType']),
			thesaurusUsageType: ThesaurusUsageTypeEnum.values.firstWhere((e) => e.index == map['ThesaurusUsageType']),
			organizationCode: map['OrganizationCode'],
			personalCode: map['PersonalCode'],
			languageCode: map['LanguageCode'],
			thesaurusUser: map['ThesaurusUser'] != null ? map['ThesaurusUser'].cast<String>().toList() : null,
			isDefault: map['IsDefault'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		if(thesaurusName != null)
			map['ThesaurusName'] = thesaurusName;
		map['ThesaurusStateType'] = thesaurusStateType.index;
		map['ThesaurusType'] = thesaurusType.index;
		map['ThesaurusUsageType'] = thesaurusUsageType.index;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(personalCode != null)
			map['PersonalCode'] = personalCode;
		if(languageCode != null)
			map['LanguageCode'] = languageCode;
		if(thesaurusUser != null)
			map['ThesaurusUser'] = thesaurusUser;
		map['IsDefault'] = isDefault;
		return map;
	}
}

class AddThesaurusRequest extends TokenRequest{
	ThesaurusDTO? thesaurus;

	AddThesaurusRequest({
		this.thesaurus,
		String? token,
	}) : super(
			token: token,
		);

	factory AddThesaurusRequest.fromJson(Map<String, dynamic> map) {
		return AddThesaurusRequest( 
			thesaurus: map['Thesaurus'] != null ? ThesaurusDTO.fromJson(map['Thesaurus']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurus != null)
			map['Thesaurus'] = thesaurus;
		return map;
	}
}

class AddThesaurusClassifyRequest extends TokenRequest{
	String? thesaurusCode;
	String? thesaurusItemName;
	String? parentItemCode;

	AddThesaurusClassifyRequest({
		this.thesaurusCode,
		this.thesaurusItemName,
		this.parentItemCode,
		String? token,
	}) : super(
			token: token,
		);

	factory AddThesaurusClassifyRequest.fromJson(Map<String, dynamic> map) {
		return AddThesaurusClassifyRequest( 
			thesaurusCode: map['ThesaurusCode'],
			thesaurusItemName: map['ThesaurusItemName'],
			parentItemCode: map['ParentItemCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		if(thesaurusItemName != null)
			map['ThesaurusItemName'] = thesaurusItemName;
		if(parentItemCode != null)
			map['ParentItemCode'] = parentItemCode;
		return map;
	}
}

class AddThesaurusContentRequest extends TokenRequest{
	String? thesaurusCode;
	String? thesaurusItemDescription;
	String? thesaurusItemConclusion;
	String? parentItemCode;

	AddThesaurusContentRequest({
		this.thesaurusCode,
		this.thesaurusItemDescription,
		this.thesaurusItemConclusion,
		this.parentItemCode,
		String? token,
	}) : super(
			token: token,
		);

	factory AddThesaurusContentRequest.fromJson(Map<String, dynamic> map) {
		return AddThesaurusContentRequest( 
			thesaurusCode: map['ThesaurusCode'],
			thesaurusItemDescription: map['ThesaurusItemDescription'],
			thesaurusItemConclusion: map['ThesaurusItemConclusion'],
			parentItemCode: map['ParentItemCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		if(thesaurusItemDescription != null)
			map['ThesaurusItemDescription'] = thesaurusItemDescription;
		if(thesaurusItemConclusion != null)
			map['ThesaurusItemConclusion'] = thesaurusItemConclusion;
		if(parentItemCode != null)
			map['ParentItemCode'] = parentItemCode;
		return map;
	}
}

class UpdateThesaurusClassifyRequest extends TokenRequest{
	String? thesaurusCode;
	String? thesaurusItemCode;
	String? thesaurusItemName;
	String? parentItemCode;

	UpdateThesaurusClassifyRequest({
		this.thesaurusCode,
		this.thesaurusItemCode,
		this.thesaurusItemName,
		this.parentItemCode,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateThesaurusClassifyRequest.fromJson(Map<String, dynamic> map) {
		return UpdateThesaurusClassifyRequest( 
			thesaurusCode: map['ThesaurusCode'],
			thesaurusItemCode: map['ThesaurusItemCode'],
			thesaurusItemName: map['ThesaurusItemName'],
			parentItemCode: map['ParentItemCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		if(thesaurusItemCode != null)
			map['ThesaurusItemCode'] = thesaurusItemCode;
		if(thesaurusItemName != null)
			map['ThesaurusItemName'] = thesaurusItemName;
		if(parentItemCode != null)
			map['ParentItemCode'] = parentItemCode;
		return map;
	}
}

class UpdateThesaurusContentRequest extends TokenRequest{
	String? thesaurusCode;
	String? thesaurusItemCode;
	String? thesaurusItemDescription;
	String? thesaurusItemConclusion;
	String? parentItemCode;

	UpdateThesaurusContentRequest({
		this.thesaurusCode,
		this.thesaurusItemCode,
		this.thesaurusItemDescription,
		this.thesaurusItemConclusion,
		this.parentItemCode,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateThesaurusContentRequest.fromJson(Map<String, dynamic> map) {
		return UpdateThesaurusContentRequest( 
			thesaurusCode: map['ThesaurusCode'],
			thesaurusItemCode: map['ThesaurusItemCode'],
			thesaurusItemDescription: map['ThesaurusItemDescription'],
			thesaurusItemConclusion: map['ThesaurusItemConclusion'],
			parentItemCode: map['ParentItemCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		if(thesaurusItemCode != null)
			map['ThesaurusItemCode'] = thesaurusItemCode;
		if(thesaurusItemDescription != null)
			map['ThesaurusItemDescription'] = thesaurusItemDescription;
		if(thesaurusItemConclusion != null)
			map['ThesaurusItemConclusion'] = thesaurusItemConclusion;
		if(parentItemCode != null)
			map['ParentItemCode'] = parentItemCode;
		return map;
	}
}

class RemoveThesaurusRequest extends TokenRequest{
	String? thesaurusCode;

	RemoveThesaurusRequest({
		this.thesaurusCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RemoveThesaurusRequest.fromJson(Map<String, dynamic> map) {
		return RemoveThesaurusRequest( 
			thesaurusCode: map['ThesaurusCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		return map;
	}
}

class RemoveThesaurusItemRequest extends TokenRequest{
	String? thesaurusCode;
	String? thesaurusItemCode;

	RemoveThesaurusItemRequest({
		this.thesaurusCode,
		this.thesaurusItemCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RemoveThesaurusItemRequest.fromJson(Map<String, dynamic> map) {
		return RemoveThesaurusItemRequest( 
			thesaurusCode: map['ThesaurusCode'],
			thesaurusItemCode: map['ThesaurusItemCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		if(thesaurusItemCode != null)
			map['ThesaurusItemCode'] = thesaurusItemCode;
		return map;
	}
}

class QueryExamListItemResult {
	String? examCode;
	String? patientName;
	String? age;
	String? sex;
	List<String>? associatedExamCodes;

	QueryExamListItemResult({
		this.examCode,
		this.patientName,
		this.age,
		this.sex,
		this.associatedExamCodes,
	});

	factory QueryExamListItemResult.fromJson(Map<String, dynamic> map) {
		return QueryExamListItemResult( 
			examCode: map['ExamCode'],
			patientName: map['PatientName'],
			age: map['Age'],
			sex: map['Sex'],
			associatedExamCodes: map['AssociatedExamCodes'] != null ? map['AssociatedExamCodes'].cast<String>().toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(examCode != null)
			map['ExamCode'] = examCode;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(age != null)
			map['Age'] = age;
		if(sex != null)
			map['Sex'] = sex;
		if(associatedExamCodes != null)
			map['AssociatedExamCodes'] = associatedExamCodes;
		return map;
	}
}

enum ExamStatusEnum {
	Wait,
	Done,
}

class QueryExamListRequest extends PageRequest{
	String? patientName;
	DateTime? startTime;
	DateTime? endTime;
	ExamStatusEnum examStatus;

	QueryExamListRequest({
		this.patientName,
		this.startTime,
		this.endTime,
		this.examStatus = ExamStatusEnum.Wait,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory QueryExamListRequest.fromJson(Map<String, dynamic> map) {
		return QueryExamListRequest( 
			patientName: map['PatientName'],
			startTime: map['StartTime'] != null ? DateTime.parse(map['StartTime']) : null,
			endTime: map['EndTime'] != null ? DateTime.parse(map['EndTime']) : null,
			examStatus: ExamStatusEnum.values.firstWhere((e) => e.index == map['ExamStatus']),
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientName != null)
			map['PatientName'] = patientName;
		if(startTime != null)
			map['StartTime'] = JsonRpcUtils.dateFormat(startTime!);
		if(endTime != null)
			map['EndTime'] = JsonRpcUtils.dateFormat(endTime!);
		map['ExamStatus'] = examStatus.index;
		return map;
	}
}

class QueryExamInfoResult {
	DateTime? createTime;
	String? deptName;
	String? patientName;
	String? patientAge;
	String? patientSex;
	List<PatientInfoExt>? patientInfoExtList;
	List<String>? associatedExamCodes;

	QueryExamInfoResult({
		this.createTime,
		this.deptName,
		this.patientName,
		this.patientAge,
		this.patientSex,
		this.patientInfoExtList,
		this.associatedExamCodes,
	});

	factory QueryExamInfoResult.fromJson(Map<String, dynamic> map) {
		return QueryExamInfoResult( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			deptName: map['DeptName'],
			patientName: map['PatientName'],
			patientAge: map['PatientAge'],
			patientSex: map['PatientSex'],
			patientInfoExtList: map['PatientInfoExtList'] != null ? (map['PatientInfoExtList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			associatedExamCodes: map['AssociatedExamCodes'] != null ? map['AssociatedExamCodes'].cast<String>().toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(deptName != null)
			map['DeptName'] = deptName;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(patientAge != null)
			map['PatientAge'] = patientAge;
		if(patientSex != null)
			map['PatientSex'] = patientSex;
		if(patientInfoExtList != null)
			map['PatientInfoExtList'] = patientInfoExtList;
		if(associatedExamCodes != null)
			map['AssociatedExamCodes'] = associatedExamCodes;
		return map;
	}
}

class QueryExamInfoRequest extends TokenRequest{
	String? examCode;

	QueryExamInfoRequest({
		this.examCode,
		String? token,
	}) : super(
			token: token,
		);

	factory QueryExamInfoRequest.fromJson(Map<String, dynamic> map) {
		return QueryExamInfoRequest( 
			examCode: map['ExamCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(examCode != null)
			map['ExamCode'] = examCode;
		return map;
	}
}

class UpdateThesaurusUserCodeRequest extends TokenRequest{
	String? thesaurusCode;

	UpdateThesaurusUserCodeRequest({
		this.thesaurusCode,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateThesaurusUserCodeRequest.fromJson(Map<String, dynamic> map) {
		return UpdateThesaurusUserCodeRequest( 
			thesaurusCode: map['ThesaurusCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		return map;
	}
}

class DeleteThesaurusUserCodeRequest extends TokenRequest{
	String? thesaurusCode;

	DeleteThesaurusUserCodeRequest({
		this.thesaurusCode,
		String? token,
	}) : super(
			token: token,
		);

	factory DeleteThesaurusUserCodeRequest.fromJson(Map<String, dynamic> map) {
		return DeleteThesaurusUserCodeRequest( 
			thesaurusCode: map['ThesaurusCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		return map;
	}
}

class GetThesaurusPageRequest extends PageRequest{
	bool? isDefault;
	String? userCode;
	ThesaurusStatusTypeEnum? thesaurusStateType;
	ThesaurusTypeEnum? thesaurusType;
	String? organizationCode;
	String? personalCode;
	String? languageCode;
	String? thesaurusName;

	GetThesaurusPageRequest({
		this.isDefault,
		this.userCode,
		this.thesaurusStateType,
		this.thesaurusType,
		this.organizationCode,
		this.personalCode,
		this.languageCode,
		this.thesaurusName,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory GetThesaurusPageRequest.fromJson(Map<String, dynamic> map) {
		return GetThesaurusPageRequest( 
			isDefault: map['IsDefault'],
			userCode: map['UserCode'],
			thesaurusStateType: map['ThesaurusStateType'] != null ? ThesaurusStatusTypeEnum.values.firstWhere((e) => e.index == map['ThesaurusStateType']) : null,
			thesaurusType: map['ThesaurusType'] != null ? ThesaurusTypeEnum.values.firstWhere((e) => e.index == map['ThesaurusType']) : null,
			organizationCode: map['OrganizationCode'],
			personalCode: map['PersonalCode'],
			languageCode: map['LanguageCode'],
			thesaurusName: map['ThesaurusName'],
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(isDefault != null)
			map['IsDefault'] = isDefault;
		if(userCode != null)
			map['UserCode'] = userCode;
		if(thesaurusStateType != null)
			map['ThesaurusStateType'] = thesaurusStateType;
		if(thesaurusType != null)
			map['ThesaurusType'] = thesaurusType;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(personalCode != null)
			map['PersonalCode'] = personalCode;
		if(languageCode != null)
			map['LanguageCode'] = languageCode;
		if(thesaurusName != null)
			map['ThesaurusName'] = thesaurusName;
		return map;
	}
}

enum ThesaurusItemTypeEnum {
	Classification,
	Contents,
}

class ThesaurusItemDTO {
	String? thesaurusItemCode;
	ThesaurusItemTypeEnum thesaurusItemType;
	String? thesaurusItemName;
	String? thesaurusItemDescription;
	String? thesaurusItemConclusion;
	String? parentItemCode;
	bool hasChildren;

	ThesaurusItemDTO({
		this.thesaurusItemCode,
		this.thesaurusItemType = ThesaurusItemTypeEnum.Classification,
		this.thesaurusItemName,
		this.thesaurusItemDescription,
		this.thesaurusItemConclusion,
		this.parentItemCode,
		this.hasChildren = false,
	});

	factory ThesaurusItemDTO.fromJson(Map<String, dynamic> map) {
		return ThesaurusItemDTO( 
			thesaurusItemCode: map['ThesaurusItemCode'],
			thesaurusItemType: ThesaurusItemTypeEnum.values.firstWhere((e) => e.index == map['ThesaurusItemType']),
			thesaurusItemName: map['ThesaurusItemName'],
			thesaurusItemDescription: map['ThesaurusItemDescription'],
			thesaurusItemConclusion: map['ThesaurusItemConclusion'],
			parentItemCode: map['ParentItemCode'],
			hasChildren: map['HasChildren'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(thesaurusItemCode != null)
			map['ThesaurusItemCode'] = thesaurusItemCode;
		map['ThesaurusItemType'] = thesaurusItemType.index;
		if(thesaurusItemName != null)
			map['ThesaurusItemName'] = thesaurusItemName;
		if(thesaurusItemDescription != null)
			map['ThesaurusItemDescription'] = thesaurusItemDescription;
		if(thesaurusItemConclusion != null)
			map['ThesaurusItemConclusion'] = thesaurusItemConclusion;
		if(parentItemCode != null)
			map['ParentItemCode'] = parentItemCode;
		map['HasChildren'] = hasChildren;
		return map;
	}
}

class GetThesaurusItemsRequest extends TokenRequest{
	String? thesaurusCode;

	GetThesaurusItemsRequest({
		this.thesaurusCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetThesaurusItemsRequest.fromJson(Map<String, dynamic> map) {
		return GetThesaurusItemsRequest( 
			thesaurusCode: map['ThesaurusCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		return map;
	}
}

class GetThesaurusClassifyByParentCodeRequest extends TokenRequest{
	String? thesaurusCode;
	String? parentItemCode;

	GetThesaurusClassifyByParentCodeRequest({
		this.thesaurusCode,
		this.parentItemCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetThesaurusClassifyByParentCodeRequest.fromJson(Map<String, dynamic> map) {
		return GetThesaurusClassifyByParentCodeRequest( 
			thesaurusCode: map['ThesaurusCode'],
			parentItemCode: map['ParentItemCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		if(parentItemCode != null)
			map['ParentItemCode'] = parentItemCode;
		return map;
	}
}

class GetThesaurusContentByParentCodeRequest extends TokenRequest{
	String? thesaurusCode;
	String? parentItemCode;

	GetThesaurusContentByParentCodeRequest({
		this.thesaurusCode,
		this.parentItemCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetThesaurusContentByParentCodeRequest.fromJson(Map<String, dynamic> map) {
		return GetThesaurusContentByParentCodeRequest( 
			thesaurusCode: map['ThesaurusCode'],
			parentItemCode: map['ParentItemCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		if(parentItemCode != null)
			map['ParentItemCode'] = parentItemCode;
		return map;
	}
}

class CopyThesaurusRequest extends TokenRequest{
	String? thesaurusCode;

	CopyThesaurusRequest({
		this.thesaurusCode,
		String? token,
	}) : super(
			token: token,
		);

	factory CopyThesaurusRequest.fromJson(Map<String, dynamic> map) {
		return CopyThesaurusRequest( 
			thesaurusCode: map['ThesaurusCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		return map;
	}
}

class CopyOrgThesaurusRequest extends TokenRequest{
	String? thesaurusCode;

	CopyOrgThesaurusRequest({
		this.thesaurusCode,
		String? token,
	}) : super(
			token: token,
		);

	factory CopyOrgThesaurusRequest.fromJson(Map<String, dynamic> map) {
		return CopyOrgThesaurusRequest( 
			thesaurusCode: map['ThesaurusCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(thesaurusCode != null)
			map['ThesaurusCode'] = thesaurusCode;
		return map;
	}
}

class SaveUserDefinedMeasureItemsRequest extends TokenRequest{

	SaveUserDefinedMeasureItemsRequest({
		String? token,
	}) : super(
			token: token,
		);

	factory SaveUserDefinedMeasureItemsRequest.fromJson(Map<String, dynamic> map) {
		return SaveUserDefinedMeasureItemsRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class SaveUserDefinedCommentItemsRequest extends TokenRequest{

	SaveUserDefinedCommentItemsRequest({
		String? token,
	}) : super(
			token: token,
		);

	factory SaveUserDefinedCommentItemsRequest.fromJson(Map<String, dynamic> map) {
		return SaveUserDefinedCommentItemsRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}


