import 'authentication.m.dart';

class StorageRequest {
	List<int>? file;
	String? authorization;

	StorageRequest({
		this.file,
		this.authorization,
	});

	factory StorageRequest.fromJson(Map<String, dynamic> map) {
		final fileData = map['File'];
		return StorageRequest( 
			file: fileData != null ? (fileData as List).map((e) => e as int).toList(): null,
			authorization: map['Authorization'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(file != null)
			map['File'] = file;
		if(authorization != null)
			map['Authorization'] = authorization;
		return map;
	}
}

class StorageServiceSettingDTO {
	String? authorization;
	String? contentType;
	String? storageUrl;

	StorageServiceSettingDTO({
		this.authorization,
		this.contentType,
		this.storageUrl,
	});

	factory StorageServiceSettingDTO.fromJson(Map<String, dynamic> map) {
		return StorageServiceSettingDTO( 
			authorization: map['Authorization'],
			contentType: map['ContentType'],
			storageUrl: map['StorageUrl'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(authorization != null)
			map['Authorization'] = authorization;
		if(contentType != null)
			map['ContentType'] = contentType;
		if(storageUrl != null)
			map['StorageUrl'] = storageUrl;
		return map;
	}
}

enum UploadFileTypeEnum {
	Unknown,
	EXE,
	APK,
	IPA,
	ZIP,
	DAT,
	RAR,
	PNG,
	ICON,
	BMP,
	JPEG,
	JPG,
	GIF,
	WEBP,
	TIFF,
	IMG,
	PDF,
	DOC,
	DOCX,
	XLS,
	XLSX,
	MP4,
	MSI,
	VID,
}

enum ApplicantTypeEnum {
	Client,
	Device,
	Management,
	ThirdParty,
	Server,
}

class FileServiceRequest extends TokenRequest{
	String? fileName;
	UploadFileTypeEnum fileTypeEnum;
	ApplicantTypeEnum applicantTypeEnum;

	FileServiceRequest({
		this.fileName,
		this.fileTypeEnum = UploadFileTypeEnum.Unknown,
		this.applicantTypeEnum = ApplicantTypeEnum.Client,
		String? token,
	}) : super(
			token: token,
		);

	factory FileServiceRequest.fromJson(Map<String, dynamic> map) {
		return FileServiceRequest( 
			fileName: map['FileName'],
			fileTypeEnum: UploadFileTypeEnum.values.firstWhere((e) => e.index == map['FileTypeEnum']),
			applicantTypeEnum: ApplicantTypeEnum.values.firstWhere((e) => e.index == map['ApplicantTypeEnum']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(fileName != null)
			map['FileName'] = fileName;
		map['FileTypeEnum'] = fileTypeEnum.index;
		map['ApplicantTypeEnum'] = applicantTypeEnum.index;
		return map;
	}
}


